/******************************************************************************

 *****************************************************************************/

#include "predef.h"
#include <stdio.h>
#include <ctype.h>
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <netinterface.h>


extern "C" {
   void UserMain(void * pd);
}

const char * AppName="OsCritObj";

OS_CRIT MyCriticalSection;
DWORD Task2Stack[USER_TASK_STK_SIZE];

/*-------------------------------------------------------------------
 HighPrioTask
 ------------------------------------------------------------------*/
void Task2( void *pdata )
{
   iprintf( "Creating Task2 OsCritObj and delaying 5 seconds\r\n" );
   {
      OSCriticalSectionObj Task2CriticalSection( MyCriticalSection );
      OSTimeDly( TICKS_PER_SECOND*5 );  // this is a blocking call
   }
   iprintf( "Exited Task2 OsCritObj. Task2 terminated.\r\n" );
}


/*-------------------------------------------------------------------
 UserMain
 ------------------------------------------------------------------*/
void UserMain(void * pd)
{
   InitializeStack();

   int EthernetInterface = GetFirstInterface();  // Get first interface identifier
   InterfaceBlock *ib = GetInterFaceBlock( EthernetInterface );  // Get interface data
   if (ib->netIP == 0)  // Check IP address for 0.0.0.0, and use DHCP if necessary
   {
      iprintf( "\r\nNo static IP address set, attempting DHCP\r\n");
      if ( GetDHCPAddress() != DHCP_OK )
         iprintf("*** DHCP Failed \r\n");
      else
         iprintf( "DHCP IP Address: ");
   }
   else
      iprintf("Static IP Address: ");
   ShowIP(ib->netIP); iprintf("\r\n");

   OSChangePrio(MAIN_PRIO);
   EnableAutoUpdate();

   OSCritInit( &MyCriticalSection );

   iprintf("Creating Task2...");
   /* Create and launch new task. Note that we selected a priority of
      MAIN_PRIO+1, which means the new task is of lower priority than UserMain.
      The implication is that if UserMain does not block, MyTask will never
      run.
   */
   if ( OSTaskCreate( Task2,
                      ( void * ) NULL,
                      ( void * ) &Task2Stack[USER_TASK_STK_SIZE],
                      ( void * ) Task2Stack,
                      MAIN_PRIO - 1 ) != OS_NO_ERR )
      iprintf( "*** Error creating task\r\n" );
   else
      iprintf("Successful\r\n");

   {  // opening scope operator

      iprintf( "Creating UserMain OsCritObj and delaying 5 seconds\r\n" );

      // Now create an OSCriticalSectionObj object. When the object gets
      // created, the constructor will be called, which will call OSCritEnter()
      OSCriticalSectionObj UserMainCriticalSection( MyCriticalSection );

      iprintf( "Exited UserMain OsCritObj. Task2 terminated.\r\n" );

   }  // destructor gets call when object goes out of scope, which calls OSCritExit()


   iprintf("Example complete. Press reset to run again\r\n");
   while(1);
   {
      OSTimeDly( TICKS_PER_SECOND*5 );  // this is a blocking call
   }

}






